ACC_bootstrap <- function(var_exp, var_obs, lon = NULL, lat = NULL,
                         lonlatbox = NULL,nrand=2000,
                         blocklen=1, method="quantile") {
# Matrix var_exp & var_obs should have dimensions (nexp/nobs,  nsdates,  
# nltimes,  nlat,  nlon).
# ACC computes the Anomaly Correlation Coefficient for each jexp in 
# 1:nexp and each jobs in 1:nobs which gives nexp x nobs ACC for each 
# startdate and each leadtime.
# A domain can be selected by providing the list of longitudes/latitudes
# (lon/lat) of the grid together with the corner of the domain:
#   lonlatbox = c(lonmin,  lonmax,  latmin,  latmax)
# 
# Args:
#   var_exp: Matrix of experimental data.
#   var_obs: Matrix of observational data,  same dimensions as var_exp except 
#            along the first dimension.
#   lon: Array of longitudes of the var_exp/var_obs grids,  optional.
#   lat: Array of latitudes of the var_exp/var_obs grids,  optional.
#   lonlatbox: Domain to select c(lonmin,  lonmax,  latmin,  latmax),  optional.
#   nrand: number of sample for the bootstrap  (default: 2000)
#   blocklen: Length of spatial blocks used for resampling the field
# 
# Returns:
#   $ACC: Matrix with c(nexp,  nobs,  nsdates,  nleadtimes,  4) dimensions.
#        The fourth dimension of length 4 corresponds to the lower limit of
#        the 95% confidence interval,  the computed ACC,  the upper limit of 
#        the 95% confidence interval and the p-value. 
#        The lower/upper limit of 95% interval and the p-values are estimated
#        with the bootstrap sampling based on Mason and Mimmack (1991)
#        if method "mason" set or,  in the default case ("quantile"),
#        simply the 2.5% and the 97.5% of the bootstrap distribution.
#        With the bootstrap technique,  it is not mandatory that
#        the real correlation is included in the confidence interval.
#        The significance of the ACC can be estimated with to different ways:
#        p-value<0.05 or a confidence interval which does not include the 0.
# 
#   $MACC: Matrix with c(nexp,  nobs,  nsdates,  nleadtimes,  4) dimensions.
#        The fourth dimension of length 4 corresponds to the lower limit of
#        the 95% confidence interval,  the computed Mean Anomaly Correlation
#        Coefficient, the upper limit of  the 95% confidence interval
#        and the p-value. 
#        The lower/upper limit of 95% interval and the p-values are estimated
#        with the bootstrap sampling based on Mason and Mimmack (1991)
#        if method "mason" set or,  in the default ("quantile") case,
#        simply the 2.5% and the 97.5% of the bootstrap distribution.
#        With the bootstrap technique it is not mandatory that the real correlation
#        is included in the confidence interval.
#        The significance of the ACC can be estimated with to different ways:
#        p-value<0.05 or a confidence interval which does not include the 0.
# 
# History:
#   1.0  #  2013-08  (C.Prodhomme,  chloe.prodhomme@ic3.cat)  #  Original code
  
  dimsvar <- dim(var_exp)
  if (length(dimsvar) != 5) {
    stop("var_exp & var_obs should have dimensions
        (nexp/nsobs, nsdates, nltimes,  nlat,  nlon) ")
  }
  for (iind in 2:length(dimsvar)) {
    if (dim(var_obs)[iind] != dimsvar[iind]) {
      stop("var_exp & var_obs must have same dimensions except the first one
          (number of experiments or number of observational datasets) ")
    }
  }
  nexp <- dimsvar[1]
  nobs <- dim(var_obs)[1]
  nsdates <- dimsvar[2]
  nltimes <- dimsvar[3]
  nlat <- dimsvar[4]
  nlon <- dimsvar[5]

  #number of blocks needed to reconstruct the field
  nblon1=floor((nlon - 1)/blocklen)+1
  nblat1=floor((nlat - 1)/blocklen)+1
  
  distriACC <- array(NA,  dim=c(nexp, nobs, nsdates, nltimes, nrand))
  distriMACC <- array(NA,  dim=c(nexp, nobs, nltimes, nrand))
  #output
  ACC <- array(NA,  dim=c(nexp, nobs, nsdates, nltimes, nrand, 3))
  MACC <- array(NA,  dim=c(nexp, nobs, nltimes, nrand))

  #bootstrap
  for (irand in 1:nrand) {
      exprand <- array(NA,
                       dim=c(nexp, nsdates, nltimes,
                         nblat1 * blocklen, nblon1 * blocklen))
      #choose of random points for exp
      vlonrand <- sample(1:(nlon - blocklen),  nblon1,  replace=T)
      vlatrand <- sample(1:(nlat - blocklen),  nblat1,  replace=T)

      #reconstruct the new random field
      #   #loops over the elements of the blocks
      for (iblock in 0:(blocklen - 1))
        {
          for (jblock in 0:(blocklen - 1))
            {
              exprand[, , , seq(1, nlat, blocklen)+jblock ,
                      seq(1, nlon, blocklen)+iblock]=
                var_exp[, , , vlatrand+jblock, vlonrand+iblock]
            }
        }

      #  #reformating in case of nexp=1

      exprand <- array(exprand[, , , 1:nlat, 1:nlon],
                       dim=c(nexp, nsdates, nltimes,
                         nlat, nlon))
      #calcultate the ACC
      #print(dim(exprand))
      ACC=ACC(exprand, var_obs, conf=FALSE)
      distriACC[, , , , irand] <- (ACC$ACC)
      distriMACC[, , ,  irand] <- (ACC$MACC)
      print(irand)
    }
  #output definition
  ACC <- array(NA,  dim=c(nexp, nobs, nsdates, nltimes, 4))
  MACC <- array(NA,  dim=c(nexp, nobs, nltimes, 4))

  #calcultate the ACC and MACC of original data
  ACCori <- ACC(var_exp, var_obs, conf = FALSE)
  ACC[, , , , 2] <- (ACCori$ACC)
  MACC[, , ,  2] <- (ACCori$MACC)


  #calculate the p-value from the bootstrap distribution
  distriACCaux=abind(distriACC, array(ACC[, , , , 2], 
    dim=c(nexp, nobs, nsdates, nltimes, 1)), along=5)
  distriMACCaux=abind(distriMACC, array(MACC[ , , ,  2], 
    dim=c(nexp, nobs, nltimes, 1)), along=4)
  
  ACC[, , , , 4] <- apply(distriACCaux, c(1, 2, 3, 4),
                          function(x)
                          sum(abs(x[1:nrand]) > abs(x[nrand+1]), na.rm=TRUE)
                          / nrand)
  MACC[, , ,  4] <- apply(distriMACCaux, c(1, 2, 3),
                          function(x)
                          sum(abs(x[1:nrand]) > abs(x[nrand+1]), na.rm=TRUE)
                          / nrand)
  
  #calcultate the confidence interval

  if(method=="mason")
    {
      #  #Calcultate the inverse CDF for a normal law

      ACCb <- apply(array(1 - ACC[, , , , 4],
                          dim=c(nexp, nobs, nsdates, nltimes)), c(1, 2, 3, 4),
                    function(x) qnorm(x))
      MACCb <- apply(array(1 - MACC[, , ,  4], dim=c(nexp, nobs, nltimes)), c(1, 2, 3),
                     function(x) qnorm(x))

      #  #975% quantile -> to get the 95% confidence intervale
      delta=qnorm(0.975)

      ACClow <- apply(ACCb, c(1, 2, 3, 4),  function(x) pnorm(2 * x - delta))
      ACChig <- apply(ACCb, c(1, 2, 3, 4),  function(x) pnorm(2 * x + delta))
      MACClow < -  apply(MACCb, c(1, 2, 3),  function(x) pnorm(2 * x - delta))
      MACChig <- apply(MACCb, c(1, 2, 3),  function(x) pnorm(2 * x + delta))


      #  #calculate the quantiles associated to this probability
      
      distriACCaux <- abind(distriACC, ACClow, ACChig, along=5)
      distriMACCaux <- abind(distriMACC, MACClow, MACChig, along=4)

      ACC[, , , , 1] <- apply(distriACCaux, c(1, 2, 3, 4),  function(x)
                              quantile(x[1:nrand],  c(x[nrand+1]), na.rm=TRUE))
      ACC[, , , , 3] <- apply(distriACCaux, c(1, 2, 3, 4),  function(x)
                              quantile(x[1:nrand],  c(x[nrand+2]), na.rm=TRUE))
      MACC[, , ,  1] <- apply(distriMACCaux, c(1, 2, 3),  function(x)
                              quantile(x[1:nrand],  c(x[nrand+1]), na.rm=TRUE))
      MACC[, , ,  3] <- apply(distriMACCaux, c(1, 2, 3),  function(x)
                              quantile(x[1:nrand],  c(x[nrand+2]), na.rm=TRUE))
      
    }else if(method=="quantile")
      {
        ACC[, , , , 3] <- apply(distriACC, c(1, 2, 3, 4),  function(x)
                                quantile(x[1:nrand],  c(0.975), na.rm=TRUE))
        ACC[, , , , 1] <- apply(distriACC, c(1, 2, 3, 4),  function(x)
                                quantile(x[1:nrand],  c(0.025), na.rm=TRUE))
        MACC[, , ,  1] <- apply(distriMACC, c(1, 2, 3),  function(x)
                                quantile(x[1:nrand],  c(0.025), na.rm=TRUE))
        MACC[, , ,  3] <- apply(distriMACC, c(1, 2, 3),  function(x)
                                quantile(x[1:nrand],  c(0.975), na.rm=TRUE))
      }else{
        print("WARNING:only quantile and mason method are available")
        print("the dimension corresponding to confident
               interval will be filled with NA")
      }

  
  invisible(list(ACC = ACC,  MACC = MACC ))

}

